#ifndef __THE_MAZE_OF_GALIOUS_2_ACTIVE_GAME
#define __THE_MAZE_OF_GALIOUS_2_ACTIVE_GAME


// ACTIVE GAME: 

#define CONTROL_LEFT	0
#define CONTROL_RIGHT	1
#define CONTROL_UP		2
#define CONTROL_DOWN	3
#define CONTROL_FIRE1	4
#define CONTROL_FIRE2	5
 

class VIEWPORT {
public:
	VIEWPORT();
	~VIEWPORT();

	SDL_Rect vp;
	int last_x,last_y;
	float zoom;
	class ActiveINTERFACE *current_interface;
};


class KEYBOARDSTATE {
public:
	KEYBOARDSTATE();
	~KEYBOARDSTATE();

	void copy(KEYBOARDSTATE *lvk);

	void cycle(void);

	bool save(FILE *fp);
	bool load(FILE *fp);	


	int n_joysticks;
	SDL_Joystick **joysticks;

	int joystick_0_pos,joystick_size;	/* start of the first joystick and joystick size */ 
	int LVK_ID;

	int k_size;							/* number of inputs */ 
	unsigned char *old_keyboard;
	unsigned char *keyboard;
	List<SDL_keysym> keyevents;
};


class DRAWCOMMAND {
public:
	int command;	/* 0 : pixel, 1 : line, 2 : box, 3 : bitmap */ 
	int x1,y1,x2,y2;	/* x2 and y2 are used for bank and bitmap in the BITMAP command */ 
	int r,g,b,a;
};


class ActiveMOG2 {
	friend class GameWindow;
	friend class GameDebugWindow;
	friend class NetworkGameWindow;

	friend class ActiveMAP;
	friend class ActiveROOM;
	friend class ActiveOBJECT;
	friend class ActiveINTERFACE;
	friend class ActiveMOG2Application;

	friend class ValueSCRIPT;
	friend class ActionSCRIPT;

public:
	ActiveMOG2(MOG2 *game);
	~ActiveMOG2(void);

	bool save(FILE *fp);
	bool load(FILE *fp);

	bool cycle(KEYBOARDSTATE **LVK,int n_LVKs);	/* Eecutes a game cycle. */ 
	void draw(SDL_Surface *sfc);				/* Draws the current state of the game	*/ 

	void reset(void);							/* Resets the game to its initial state. */ 

	ActiveOBJECT *add_player(MOG2_INTERFACE *interf,ROOM *room,char *character,char *playername,int x,int y,SDL_Rect *vp,float zoom,int LVK_ID);
	ActiveOBJECT *add_player(MOG2_INTERFACE *interf,ROOM *room,char *character,char *playername,SDL_Rect *vp,float zoom,int LVK_ID);

	ActiveOBJECT *find_object(int ID);
	OBJECT *find_object(char *name);
	bool find_object(ActiveOBJECT *o);
	ActiveOBJECT *find_activeinstance(ActiveOBJECT *o,ActiveMAP *m);
	void remove_focusing_references(ActiveOBJECT *o);	/* Looks through all the viewports and interfaces	*/ 
														/* looking for references to 'o', and deletes them.	*/ 

	bool focusing_object(ActiveOBJECT *o);

	int get_variable_value(char *vname,int *ival,char **sval,ActiveOBJECT **oval);
	void set_variable_value(char *vname,int value);
	void set_variable_value(char *vname,char *value);
	void set_variable_value(char *vname,ActiveOBJECT *value);

	/* Hypothetical execution: */ 
	bool hypothetic_cycle(KEYBOARDSTATE **LVK,int n_LVKs,int cycle_num);
	bool hypothetic_test_and_fix(KEYBOARDSTATE **LVK,int n_LVKs,int cycle_num);

	int get_current_cycle(void);
	void add_sample(class PLAYSAMPLE *s);

private:

	void init_map(MAP *m);

	ActiveROOM *add_room(ROOM *r);


	class ActiveMOG2Application *app;

	// Game state: 
	List<VARIABLE> game_variables;
	MOG2 *active_game;
	bool paused;

	List<ActiveOBJECT> player_objects;
	List<char> player_names;
	bool deleted_objects;

	List<ActiveMAP> active_maps;
	List<VIEWPORT> viewports;

	class ActionSCRIPT *game_action;
};


#endif

